跳到论文正文
W Papers独立论文阅读器 arXiv:2601.05272 论文正文 计算阅读时间 · 0%
引用这篇论文根据当前论文元数据生成,正式投稿前建议核对。

    
  
License: CC BY 4.0
arXiv:2601.05272v1 [cs.SC] 18 Dec 2025

A Rank 23 Algorithm for Multiplying 3×33\times 3 Matrices with an Arithmetic Complexity of 59

Erik Mårtensson Lund University, Lund, Sweden {erik.martensson,paul.stankovski_wagner}@eit.lth.se Advenica AB, Malmö, Sweden Paul Stankovski Wagner Lund University, Lund, Sweden {erik.martensson,paul.stankovski_wagner}@eit.lth.se Joshua Stapleton Department of Mathematics, Imperial College London, UK jbs123@ic.ac.uk
Abstract

In 1969 Strassen showed surprisingly that it is possible to multiply two 2×22\times 2 matrices using seven multiplications and 18 additions, instead of the naive eight multiplications and four additions. The number of additions was later reduced to 15. Karstadt and Schwartz further reduced the number of additions to 12 using a change-of-basis method. Both the number of multiplications and the number of additions have been shown to be optimal for the 2×22\times 2 case.

For multiplying 3×33\times 3 matrices, the lowest number of multiplications found so far is 23. Using 23 multiplications, Schwart et al. showed how to reduce the number of additions to 61 using a change-of-basis method. Mårtensson and Stankovski Wagner showed how to achieve 62 additions, without changing basis. Using the optimization method by Mårtensson and Stankovski Wagner, Stapleton found an algorithm requiring only 60 additions. In this work we continue to combine the methods of Mårtensson, Stankovski Wagner and Stapleton, finding an algorithm requiring only 59 additions, still without a basis change. Technical details on the method and tools used for finding this scheme, and a discussion on the impact of this discovery, will come in an upcoming publication.

1 Introduction

Multiplying 2×22\times 2 matrices directly from the definition requires eight multiplications and four additions. In 1969 Strassen [STR69] showed how to reduce the number of multiplications to seven, at the cost of increasing the number of additions to 18111When referring to additions, we also implicitly include both additions and subtractions.. The number of multiplications is also called the rank of the algorithm. The rank seven was shown to be optimal in [WIN71].

For rank 7 algorithms for multiplying 2×22\times 2 matrices, the number of additions was reduced to 15 in [BSH95]. The number of additions required is also referred to as the arithmetic complexity of the algorithm. Using a change of basis, Karstadt and Schwartz improved this number to 12 and showed it to be optimal in [KS20]. In terms of computational complexity, performing this change of basis introduces lower-order terms that are small compared to the cost of performing an addition.

Now consider multiplying an n×mn\times m matrix by an m×km\times k matrix. For most of the matrix dimensions larger than that of 2×22\times 2, both the optimal rank and the optimal arithmetic complexity of matrix multiplication are not known. For a collection of the schemes found for a large number of dimensions, see [SED25].

For multiplying 3×33\times 3 matrices, Laderman found a rank 23 algorithm by hand in 1976 [LAD76]. After almost half a century of applying increasingly sophisticated search methods, this rank has not been improved. In this work, we introduce a new record for the number of additions for a rank 23 algorithm for multiplying 3×33\times 3 matrices.

In terms of the number of additions required for rank 23 algorithms for multiplying 3×33\times 3 matrices, Smirnov found an algorithm that naively requires 84 additions [SMI13]. Smirnov pointed out that the number of additions for his algorithm can be lowered, but he did not perform such an optimization. Karstadt and Schwartz reduced the number of additions to 75 in [KS20]. Beniamini et al. reduced this number further to 61 in [BCH+20]. Without performing a change of basis, Mårtensson and Stankovski Wagner showed how to perform Laderman’s original algorithm using only 62 additions, almost matching the result of Beniamini et al. [MS25].

Recently, Stapleton developed a method for generating low-rank matrix multiplication schemes using a neural network [STA25]. By generating many schemes of rank 23 and reducing the number of additions using the implementations of Mårtensson and Stankovski Wagner, he found a 60-addition algorithm.

We further combined Stapleton’s method to generate fast matrix multiplication schemes with Mårtensson and Stankovski Wagner’s method for reducing the number of additions to find a scheme requiring only 59 additions. The naive (unreduced) form of the scheme uses 110 additions and is specified in Table 1. Our optimized scheme is specified in Table 2, requiring only 59 additions. We use the compact indexing notation

(C0C1C2C3C4C5C6C7C8)=𝐂=𝐀𝐁=(A0A1A2A3A4A5A6A7A8)​(B0B1B2B3B4B5B6B7B8)\begin{pmatrix}C_{0}&C_{1}&C_{2}\\ C_{3}&C_{4}&C_{5}\\ C_{6}&C_{7}&C_{8}\end{pmatrix}=\mathbf{C}=\mathbf{A}\mathbf{B}=\begin{pmatrix}A_{0}&A_{1}&A_{2}\\ A_{3}&A_{4}&A_{5}\\ A_{6}&A_{7}&A_{8}\end{pmatrix}\begin{pmatrix}B_{0}&B_{1}&B_{2}\\ B_{3}&B_{4}&B_{5}\\ B_{6}&B_{7}&B_{8}\end{pmatrix}.

Finally, for convenience, in the Appendix we have specified the 59-algorithm on a form readable by the the software implementations of Mårtensson [MÅR24] and Stankovski Wagner [STA24] in  Table 3.

More technical details on how the scheme was found, tools and methods used, as well as implications of the finding, will be included in an upcoming publication.

M0\displaystyle M_{0} =\displaystyle= A1​B4\displaystyle\phantom{(}A_{1}B_{4} M1\displaystyle M_{1} =\displaystyle= A0​B1\displaystyle\phantom{(}A_{0}B_{1} M2\displaystyle M_{2} =\displaystyle= (A3−A4−A5+A6−A7−A8)​B8\displaystyle(A_{3}-A_{4}-A_{5}+A_{6}-A_{7}-A_{8})B_{8} M3\displaystyle M_{3} =\displaystyle= (A2−A8)​B8\displaystyle(A_{2}-A_{8})B_{8} M4\displaystyle M_{4} =\displaystyle= (A3−A5+A6−A8)​B6\displaystyle(A_{3}-A_{5}+A_{6}-A_{8})B_{6} M5\displaystyle M_{5} =\displaystyle= A2​B7\displaystyle\hphantom{(}A_{2}B_{7} M6\displaystyle M_{6} =\displaystyle= A0​B0\displaystyle\hphantom{(}A_{0}B_{0} M7\displaystyle M_{7} =\displaystyle= (A1−A7)​(B2+B5)\displaystyle(A_{1}-A_{7})(B_{2}+B_{5}) M8\displaystyle M_{8} =\displaystyle= A5​(B4+B5−B6−B7−B8)\displaystyle\phantom{(}A_{5}(B_{4}+B_{5}-B_{6}-B_{7}-B_{8}) M9\displaystyle M_{9} =\displaystyle= A4​B3\displaystyle\phantom{(}A_{4}B_{3} M10\displaystyle M_{10} =\displaystyle= (A0−A1−A6+A7)​B2\displaystyle(A_{0}-A_{1}-A_{6}+A_{7})B_{2} M11\displaystyle M_{11} =\displaystyle= (A3+A6−A8)​(B1+B2+B4+B5−B6)\displaystyle(A_{3}+A_{6}-A_{8})(B_{1}+B_{2}+B_{4}+B_{5}-B_{6}) M12\displaystyle M_{12} =\displaystyle= (A1+A6−A7)​(B2+B4+B5)\displaystyle(A_{1}+A_{6}-A_{7})(B_{2}+B_{4}+B_{5}) M13\displaystyle M_{13} =\displaystyle= A1​B3\displaystyle\phantom{(}A_{1}B_{3} M14\displaystyle M_{14} =\displaystyle= (A1+A3+A6−A7)​(B2−B3+B5)\displaystyle(A_{1}+A_{3}+A_{6}-A_{7})(B_{2}-B_{3}+B_{5}) M15\displaystyle M_{15} =\displaystyle= (A3−A4+A6−A7)​(B3−B5+B8)\displaystyle(A_{3}-A_{4}+A_{6}-A_{7})(B_{3}-B_{5}+B_{8}) M16\displaystyle M_{16} =\displaystyle= (A3−A4−A5+A6−A7)​(B4+B5−B8)\displaystyle(A_{3}-A_{4}-A_{5}+A_{6}-A_{7})(B_{4}+B_{5}-B_{8}) M17\displaystyle M_{17} =\displaystyle= A3​(B0+B1+B2+B4+B5)\displaystyle\phantom{(}A_{3}(B_{0}+B_{1}+B_{2}+B_{4}+B_{5}) M18\displaystyle M_{18} =\displaystyle= (A3−A4−A5)​(B4+B5)\displaystyle(A_{3}-A_{4}-A_{5})(B_{4}+B_{5}) M19\displaystyle M_{19} =\displaystyle= A8​(B1+B2+B4+B5+B7)\displaystyle\phantom{(}A_{8}(B_{1}+B_{2}+B_{4}+B_{5}+B_{7}) M20\displaystyle M_{20} =\displaystyle= A2​B6\displaystyle\phantom{(}A_{2}B_{6} M21\displaystyle M_{21} =\displaystyle= (A6−A8)​(B1+B2+B4+B5)\displaystyle(A_{6}-A_{8})(B_{1}+B_{2}+B_{4}+B_{5}) M22\displaystyle M_{22} =\displaystyle= (A3+A6)​(B0−B2+B3−B5+B6)\displaystyle(A_{3}+A_{6})(B_{0}-B_{2}+B_{3}-B_{5}+B_{6}) C0\displaystyle C_{0} =\displaystyle= M6+M13+M20\displaystyle\phantom{-}M_{6}+M_{13}+M_{20} C1\displaystyle C_{1} =\displaystyle= M0+M1+M5\displaystyle\phantom{-}M_{0}+M_{1}+M_{5} C2\displaystyle C_{2} =\displaystyle= −M0−M2+M3+M10+M12−M16+M18\displaystyle-M_{0}-M_{2}+M_{3}+M_{10}+M_{12}-M_{16}+M_{18} C3\displaystyle C_{3} =\displaystyle= −M4+M9−M11+M17+M21\displaystyle-M_{4}+M_{9}-M_{11}+M_{17}+M_{21} C4\displaystyle C_{4} =\displaystyle= −M0+M4−M8−M9+M11+M12−M13−M14−M15−M16−M21\displaystyle-M_{0}+M_{4}-M_{8}-M_{9}+M_{11}+M_{12}-M_{13}-M_{14}-M_{15}-M_{16}-M_{21} C5\displaystyle C_{5} =\displaystyle= M0+M9−M12+M13+M14+M15+M16−M18\displaystyle\phantom{-}M_{0}+M_{9}-M_{12}+M_{13}+M_{14}+M_{15}+M_{16}-M_{18} C6\displaystyle C_{6} =\displaystyle= −M7+M11+M13+M14−M17−M21+M22\displaystyle-M_{7}+M_{11}+M_{13}+M_{14}-M_{17}-M_{21}+M_{22} C7\displaystyle C_{7} =\displaystyle= M0+M7−M12+M19+M21\displaystyle\phantom{-}M_{0}+M_{7}-M_{12}+M_{19}+M_{21} C8\displaystyle C_{8} =\displaystyle= −M0−M2−M7+M12−M16+M18\displaystyle-M_{0}-M_{2}-M_{7}+M_{12}-M_{16}+M_{18}

Table 1: Fast matrix multiplication algorithm with 110 additions (before reduction). Here we use compact indexing notation. The algorithm is specified in implementation order. Note that negation (negative leading term) is entirely avoided in implementations by rearranging terms.

t0\displaystyle t_{0} =\displaystyle= A3+A6\displaystyle A_{3}+A_{6} t1\displaystyle t_{1} =\displaystyle= A1−A7\displaystyle A_{1}-A_{7} t2\displaystyle t_{2} =\displaystyle= A4+A5\displaystyle A_{4}+A_{5} t3\displaystyle t_{3} =\displaystyle= A7−t0\displaystyle A_{7}-t_{0} t4\displaystyle t_{4} =\displaystyle= A6+t1\displaystyle A_{6}+t_{1} t5\displaystyle t_{5} =\displaystyle= A8−t0\displaystyle A_{8}-t_{0} t6\displaystyle t_{6} =\displaystyle= t2+t3\displaystyle t_{2}+t_{3} u0\displaystyle u_{0} =\displaystyle= B2+B5\displaystyle B_{2}+B_{5} u1\displaystyle u_{1} =\displaystyle= B4+u0\displaystyle B_{4}+u_{0} u2\displaystyle u_{2} =\displaystyle= B1+u1\displaystyle B_{1}+u_{1} u3\displaystyle u_{3} =\displaystyle= B4+B5\displaystyle B_{4}+B_{5} u4\displaystyle u_{4} =\displaystyle= B3−u0\displaystyle B_{3}-u_{0} u5\displaystyle u_{5} =\displaystyle= B8−u3\displaystyle B_{8}-u_{3} M0\displaystyle M_{0} =\displaystyle= A1​B4\displaystyle\phantom{(}A_{1}B_{4} M1\displaystyle M_{1} =\displaystyle= A0​B1\displaystyle\phantom{(}A_{0}B_{1} M2\displaystyle M_{2} =\displaystyle= (A8+t6)​B8\displaystyle(A_{8}+t_{6})B_{8} M3\displaystyle M_{3} =\displaystyle= (A2−A8)​B8\displaystyle(A_{2}-A_{8})B_{8} M4\displaystyle M_{4} =\displaystyle= (A5+t5)​B6\displaystyle(A_{5}+t_{5})B_{6} M5\displaystyle M_{5} =\displaystyle= A2​B7\displaystyle\phantom{(}A_{2}B_{7} M6\displaystyle M_{6} =\displaystyle= A0​B0\displaystyle\phantom{(}A_{0}B_{0} M7\displaystyle M_{7} =\displaystyle= t1​u0\displaystyle\phantom{(}t_{1}u_{0} M8\displaystyle M_{8} =\displaystyle= A5​(B6+B7+u5)\displaystyle\phantom{(}A_{5}(B_{6}+B_{7}+u_{5}) M9\displaystyle M_{9} =\displaystyle= A4​B3\displaystyle\phantom{(}A_{4}B_{3} M10\displaystyle M_{10} =\displaystyle= (A0−t4)​B2\displaystyle(A_{0}-t_{4})B_{2} M11\displaystyle M_{11} =\displaystyle= t5​(B6−u2)\displaystyle\phantom{(}t_{5}(B_{6}-u_{2}) M12\displaystyle M_{12} =\displaystyle= t4​u1\displaystyle\phantom{(}t_{4}u_{1} M13\displaystyle M_{13} =\displaystyle= A1​B3\displaystyle A_{1}B_{3} M14\displaystyle M_{14} =\displaystyle= (t0+t1)​u4\displaystyle(t_{0}+t_{1})u_{4} M15\displaystyle M_{15} =\displaystyle= (A4+t3)​(B3−B5+B8)\displaystyle(A_{4}+t_{3})(B_{3}-B_{5}+B_{8}) M16\displaystyle M_{16} =\displaystyle= t6​u5\displaystyle\phantom{(}t_{6}u_{5} M17\displaystyle M_{17} =\displaystyle= A3​(B0+u2)\displaystyle\phantom{(}A_{3}(B_{0}+u_{2}) M18\displaystyle M_{18} =\displaystyle= (A3−t2)​u3\displaystyle(A_{3}-t_{2})u_{3} M19\displaystyle M_{19} =\displaystyle= A8​(B7+u2)\displaystyle\phantom{(}A_{8}(B_{7}+u_{2}) M20\displaystyle M_{20} =\displaystyle= A2​B6\displaystyle\phantom{(}A_{2}B_{6} M21\displaystyle M_{21} =\displaystyle= (A6−A8)​u2\displaystyle(A_{6}-A_{8})u_{2} M22\displaystyle M_{22} =\displaystyle= t0​(B0+B6+u4)\displaystyle\phantom{(}t_{0}(B_{0}+B_{6}+u_{4}) v0\displaystyle v_{0} =\displaystyle= M0−M12\displaystyle\phantom{-}M_{0}-M_{12} v1\displaystyle v_{1} =\displaystyle= M16+v0\displaystyle\phantom{-}M_{16}+v_{0} v2\displaystyle v_{2} =\displaystyle= M11−M21\displaystyle\phantom{-}M_{11}-M_{21} v3\displaystyle v_{3} =\displaystyle= M14−M13\displaystyle\phantom{-}M_{14}-M_{13} v4\displaystyle v_{4} =\displaystyle= M18−v1\displaystyle\phantom{-}M_{18}-v_{1} v5\displaystyle v_{5} =\displaystyle= M2+v4\displaystyle\phantom{-}M_{2}+v_{4} v6\displaystyle v_{6} =\displaystyle= M4+M9\displaystyle\phantom{-}M_{4}+M_{9} v7\displaystyle v_{7} =\displaystyle= M15+v3\displaystyle\phantom{-}M_{15}+v_{3} v8\displaystyle v_{8} =\displaystyle= M17−v2\displaystyle\phantom{-}M_{17}-v_{2} C0\displaystyle C_{0} =\displaystyle= M6+M13+M20\displaystyle\phantom{-}M_{6}+M_{13}+M_{20} C1\displaystyle C_{1} =\displaystyle= M0+M1+M5\displaystyle\phantom{-}M_{0}+M_{1}+M_{5} C2\displaystyle C_{2} =\displaystyle= M3+M10+v5\displaystyle\phantom{-}M_{3}+M_{10}+v_{5} C3\displaystyle C_{3} =\displaystyle= v6+v8\displaystyle\phantom{-}v_{6}+v_{8} C4\displaystyle C_{4} =\displaystyle= M8−v1+v2−v6+v7\displaystyle\phantom{-}M_{8}-v_{1}+v_{2}-v_{6}+v_{7} C5\displaystyle C_{5} =\displaystyle= M9−v4−v7\displaystyle\phantom{-}M_{9}-v_{4}-v_{7} C6\displaystyle C_{6} =\displaystyle= −M7+M22−v3−v8\displaystyle-M_{7}+M_{22}-v_{3}-v_{8} C7\displaystyle C_{7} =\displaystyle= M7+M19+M21+v0\displaystyle\phantom{-}M_{7}+M_{19}+M_{21}+v_{0} C8\displaystyle C_{8} =\displaystyle= −M7+v5\displaystyle-M_{7}+v_{5}

Table 2: Fast matrix multiplication algorithm reduced to 59 additions with Greedy Vanilla from [MS25]. Greedy Potential yields no improvement. Here we use compact indexing notation. The algorithm is specified in implementation order. Note that negation (negative leading term) is entirely avoided in implementations by rearranging terms.

References

  • [BCH+20] G. Beniamini, N. Cheng, O. Holtz, E. Karstadt, and O. Schwartz (2020) Sparsifying the Operators of Fast Matrix Multiplication Algorithms. External Links: 2008.03759 Cited by: §1.
  • [BSH95] N. H. Bshouty (1995) On the additive complexity of 2 × 2 matrix multiplication. Information Processing Letters 56 (6), pp. 329–335. External Links: ISSN 0020-0190, Document, Link Cited by: §1.
  • [KS20] E. Karstadt and O. Schwartz (2020-01) Matrix Multiplication, a Little Faster. J. ACM 67 (1). External Links: ISSN 0004-5411, Link, Document Cited by: §1, §1.
  • [LAD76] J. D. Laderman (1976) A Noncommutative Algorithm for Multiplying 3×33\times 3 Matrices Using 23 Multiplications. Bulletin of the American Mathematical Society 82 (1), pp. 126 – 128. Cited by: §1.
  • [MS25] E. Mårtensson and P. Stankovski Wagner (2025) The number of the beast: reducing additions in fast matrix multiplication algorithms for dimensions up to 666. In 2025 Proceedings of the Conference on Applied and Computational Discrete Algorithms (ACDA), pp. 47–60. External Links: Document, Link, https://epubs.siam.org/doi/pdf/10.1137/1.9781611978759.4 Cited by: Table 2, §1.
  • [MÅR24] E. Mårtensson (2024) GitHub – Python code repository fmm_add_reduction. Note: Available at
  • [11] github.com/ErikMaartensson/fmm_add_reduction, last accessed on July 28, 2026
  • External Links: Link Cited by: Table 3, §1.
  • [SED25] A. Sedoglavic (2025) Yet another catalogue of fast matrix multiplication algorithms.. Note: https://fmm.univ-lille.fr/ External Links: Link Cited by: §1.
  • [SMI13] A. Smirnov (2013) The Bilinear Complexity and Practical Algorithms for Matrix Multiplication. Comput. Math. and Math. Phys. 53, pp. 1781–1795. External Links: Link, Document Cited by: §1.
  • [STA24] P. Stankovski Wagner (2024) GitHub – C code repository fmm_add_reduction. Note: Available at
  • [13] github.com/werekorren/fmm_add_reduction, last accessed on July 28, 2026
  • External Links: Link Cited by: Table 3, §1.
  • [STA25] J. Stapleton (2025) A 60-addition, rank-23 scheme for exact 3x3 matrix multiplication. External Links: 2508.03857, Link Cited by: §1.
  • [STR69] V. Strassen (1969-08) Gaussian elimination is not optimal. Numer. Math. 13 (4), pp. 354–356. External Links: ISSN 0029-599X, Link, Document Cited by: §1.
  • [WIN71] S. Winograd (1971) On multiplication of 2 × 2 matrices. Linear Algebra and its Applications 4 (4), pp. 381–388. External Links: ISSN 0024-3795, Document, Link Cited by: §1.

Appendix A The 59-algorithm in File Format

For convenience and ease of use, we include our 59-algorithm in file format too.

0 1 0 0 0 0 1 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 -1 0 0 -1 0 1 -1 1 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 -1 0 0 0 0 1 0 1 0 0 0 0 0 0 -1 0 0 1 1 1 -1 1 0 0 0 -1 0 0 -1 0 0 0 0 0 0 1 0 0 0 0 0 -1 -1 0 -1 0 0 0 0 0 0 -1 0 -1 0 0 0 1 0 0 0 0 0 0 0 -1 0 -1 0 0 0 0 0 0 1 0 1 0 0 0 0 0 -1 -1 1 0 1 1 1 0 0 0 0 -1 -1 0 0 -1 0 0 0 0 1 0 0 1 0 -1 0 -1 -1 -1 0 0 0 0 0 0 0 0 -1 -1 -1 0 0 0 0 0 0 1 0 0 0 0 0 0 0 -1 0 1 0 # 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 1 0 1 0 0 0 0 0 0 0 0 0 -1 0 0 0 0 0 1 0 -1 0 -1 0 0 0 0 0 0 0 0 -1 0 0 -1 -1 1 0 1 0 0 1 0 -1 0 -1 -1 0 0 0 0 0 0 0 0 0 1 0 0 0 1 -1 -1 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 1 0 0 -1 1 0 0 0 1 1 -1 -1 0 -1 0 0 0 0 0 0 0 0 -1 1 0 0 -1 1 0 1 1 1 1 -1 -1 0 -1 -1 0 0 0 0 1 0 0 0 -1 0 0 1 0 0 0 0 0 0 0 0 -1 0 1 0 0 0 0 0 1 0 0 -1 0 0 0 0 0 0 0 0 0 0 -1 0 0 0 0 0 1 1 0 0 0 0 -1 0 0 0 0 0 0 -1 -1 0 0 0 0 0 0 # 0 0 0 0 0 0 1 0 0 0 0 0 0 -1 0 0 0 0 0 0 1 0 0 1 1 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 -1 0 -1 1 0 0 0 0 0 0 -1 0 1 0 0 0 -1 0 -1 0 0 0 0 0 0 0 0 -1 0 0 0 0 1 0 -1 0 0 0 0 0 -1 0 0 0 1 0 -1 0 0 0 1 0 0 0 -1 -1 0 1 1 1 -1 1 -1 0 0 0 0 -1 0 1 0 0 0 0 0 0 0 0 1 0 0 -1 -1 1 -1 1 0 1 0 0 0 0 0 0 0 0 0 0 0 -1 0 0 0 1 0 -1 1 0 0 1 0 0 0 -1 -1 1 0 0 0 0 0 0 1 0 0 0 0 -1 0 0 0 0 0 0 1 0 1 0 -1 0 -1 0 0 0 0 -1 0 0 0 0 1 0 0 0 -1 0 -1 0 0 0 0

Table 3: The 59-algorithm written in a format that is readable by the implementations of Mårtensson [MÅR24] and Stankovski Wagner [STA24].